@@ -9,7 +9,6 @@ const EDPK_PREFIX: [u8; 4] = [13, 15, 37, 217];
 const EDSIG_PREFIX: [u8; 5] = [9, 245, 205, 134, 18];
 const SPSIG_PREFIX: [u8; 5] = [13, 115, 101, 19, 63];
 const P2SIG_PREFIX: [u8; 4] = [54, 240, 44, 52];
-const SPSIG: &str = "4sLJ";
 
 // use crate::did::{VerificationMethod, VerificationMethodMap};
 use crate::caip10::BlockchainAccountId;
@@ -326,9 +325,19 @@ impl LinkedDataProofs {
                     .await;
             }
             Algorithm::ES256K => {
+                if let Some(ref vm) = options.verification_method {
+                    // TODO: resolve the VM to get type, rather than comparing the string directly
+                    if (vm.starts_with("did:tz:") || vm.starts_with("did:pkh:tz:"))
+                        && vm.ends_with("#TezosMethod2021")
+                    {
+                        return TezosSignature2021
+                            .prepare(document, options, public_key)
+                            .await;
+                    }
+                }
                 return EcdsaSecp256k1Signature2019
                     .prepare(document, options, public_key)
-                    .await
+                    .await;
             }
             Algorithm::ES256KR => {
                 if let Some(ref vm) = options.verification_method {
@@ -1124,21 +1133,8 @@ async fn micheline_from_document_and_options(
     let sigopts_dataset = proof.to_dataset_for_signing(Some(document)).await?;
     let sigopts_dataset_normalized = urdna2015::normalize(&sigopts_dataset)?;
     let sigopts_normalized = sigopts_dataset_normalized.to_nquads()?;
-    let string_bytes = [
-        "Tezos Signed Message: ".to_string(),
-        sigopts_normalized,
-        doc_normalized,
-    ]
-    .join("\n")
-    .as_bytes()
-    .to_vec();
-    const BYTES_PREFIX: [u8; 2] = [05, 01];
-    let data = [
-        BYTES_PREFIX.to_vec(),
-        string_bytes.len().to_be_bytes().to_vec(),
-        string_bytes,
-    ]
-    .concat();
+    let msg = ["", &sigopts_normalized, &doc_normalized].join("\n");
+    let data = crate::tzkey::encode_tezos_signed_message(&msg)?;
     Ok(data)
 }
 
@@ -1249,22 +1245,8 @@ impl ProofSuite for TezosSignature2021 {
             },
             None => None,
         };
-        let sig = bs58::decode(&sig_bs58).with_check(None).into_vec()?;
-        let (algorithm, sig) = if sig_bs58.starts_with("edsig") {
-            (Algorithm::EdDSA, sig[5..].to_vec())
-        } else if sig_bs58.starts_with(SPSIG) {
-            let alg = if proof_jwk_opt.is_some() {
-                Algorithm::ES256K
-            } else {
-                Algorithm::ES256KR
-            };
-            (alg, sig[5..].to_vec())
-        } else if sig_bs58.starts_with("p2sig") {
-            (Algorithm::ES256, sig[4..].to_vec())
-        } else {
-            return Err(Error::SignaturePrefix);
-        };
 
+        let (algorithm, sig) = crate::tzkey::decode_tzsig(sig_bs58)?;
         let vm = resolve_vm(&verification_method, resolver).await?;
         if vm.type_ != "TezosMethod2021" {
             return Err(Error::VerificationMethodMismatch);
@@ -1291,8 +1273,10 @@ impl ProofSuite for TezosSignature2021 {
                 // VM does not have publicKeyJwk: either proof must have public key or algorithm must support
                 // public key recovery.
                 if let Some(proof_jwk) = proof_jwk_opt {
-                    // Proof has public key: verify it with blockchainAccountId.
+                    // Proof has public key: verify it with blockchainAccountId,
                     account_id.verify(&proof_jwk)?;
+                    // and verify the signature.
+                    crate::jws::verify_bytes(algorithm, &data, &proof_jwk, &sig)?;
                 } else {
                     // Proof has no public key: use public key recovery,
                     let recovered_jwk = crate::jws::recover(algorithm, &data, &sig)?;
